import 'package:xml/xml.dart';

import '../classes.dart';

class DeviceQueryRequest extends CuppsBase {
  final String deviceName;
  DeviceQueryRequest({
    required int messageID,
    required this.deviceName,
    String? xmlns,
    String? xsi,
  }) : super(messageID: messageID, messageName: "deviceQueryRequest");

  factory DeviceQueryRequest.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String messageID = document.rootElement.attributes.firstWhere((p0) => p0.name.toString() == "messageID").value;
    String deviceName = document.rootElement.childElements
        .firstWhere((element) => element.name.toString() == "deviceQueryRequest")
        .attributes
        .firstWhere((p0) => p0.name.toString() == "deviceName",orElse: ()=>orElseEmpty())
        .value;
    return DeviceQueryRequest(messageID: int.parse(messageID), deviceName: deviceName);
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName deviceName="$deviceName"/>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName ";
  }
}
